#include <assert.h>
#include <stdexcept>
#include <cstdio>
#include <qtextbrowser.h>
#include <kapplication.h>
#include <ktextbrowser.h>
#include <klocale.h>
#include <klistview.h>
#include <kpushbutton.h>
#include <qtimer.h>
#include "about.h"
#include "maindialog.h"
#include "about.h"
#include "helper.h"
#include "wait.h"
#include "info.h"
#include "yesno.h"
#include <PackageTree.h>
#include <Package.h>
#include <Transaction.h>
#include <TreeDir.h>
#include <list>
#include <string>
using namespace std;

extern Wait *wait;
extern PackageTree *ptree;
extern MainDialog *mainwin;
extern KApplication *app;
extern About *abt;
extern Info *info;
extern YesNo *yesno;

MainDialog::MainDialog(QWidget* parent, const char* name, WFlags fl)
		: maindlg(parent,name,fl), selpkg(NULL), t(NULL)
	{}

MainDialog::~MainDialog()
	{}

/*$SPECIALIZATION$*/
void MainDialog::showabout()
{
	abt->hide();
	abt->show();
}

void MainDialog::fillup()
{
	ptree=new PackageTree;
	try
	{
		ptree->fillup();
		ptree->check_installed();
		fill_groups();
		wait->hide();
		mainwin->show();
	}
	catch(runtime_error e)
	{
		info->text->setText(I18N_NOOP(e.what()));
		wait->hide();
		info->show();
		connect((QObject*)info->button, SIGNAL(clicked()), app, SLOT(quit()));
	}
}

void MainDialog::fill_groups()
{
	grouptree->clear();
	
	TreeDir root=ptree->root_group();
	list<TreeDir> g=root.get_groups();
	list<TreeDir>::iterator it;
	
	for(it=g.begin(); it!=g.end(); it++)
	{
		grouptree->insertItem(generate_group_item(*it, grouptree, it->getname()));
	}
}

void MainDialog::fill_pkglist(const std::string& group)
{
	pkglist->clear();

	TreeDir& g=ptree->find_group(group);
	list<Package>& p=g.get_packages();
	list<Package>::iterator it;
	
	for(it=p.begin(); it!=p.end(); it++)
	{
		QListViewItem *item=new QListViewItem(pkglist);
		
		item->setText(0, it->getname());
		item->setText(2, it->getsummary());
		if(it->isinstalled())
			item->setText(1, "i");
		else
			item->setText(1, "");

		pkglist->insertItem(item);
	}
}

QListViewItem *MainDialog::generate_group_item(TreeDir& td, QListView *p, const string &name)
{
	QListViewItem *item=new QListViewItem(p);
	
	item->setText(0, td.getname());
	
	list<TreeDir>& g=td.get_groups();
	list<TreeDir>::iterator it;
	
	for(it=g.begin(); it!=g.end(); it++)
		item->insertItem(generate_group_item(*it, (QListView*)item, name+"/"+it->getname()));

	return item;
}

void MainDialog::clickedgroup(QListViewItem *item)
{
	if(!item)
		return;
	
	selgrp=getgroupname(item);
	
	fill_pkglist(selgrp);
}

void MainDialog::clickedpackage(QListViewItem *item)
{
	if(!item)
		return;
	
	Package& p=ptree->find_package(item->text(0));
	selpkg=&p;
	
	string dsc;
	
	dsc+="<table><tr><td><b>Name: </b></td><td>";
	dsc+=p.getname();
	dsc+="</td></tr><tr><td><b>Version: </b></td><td>";
	dsc+=p.getversion();
	dsc+="</td></tr><tr><td><b>Release: </b></td><td>";
	dsc+=p.getrelease();
	dsc+="</td></tr><tr><td><b>Summary: </b></td><td>";
	dsc+=p.getsummary();
	dsc+="</td></tr><tr><td><b>License: </b></td><td>";
	dsc+=p.getlicense();
	if(p.geturl()!="")
	{
		dsc+="</td></tr><tr><td><b>URL: </b></td><td>";
		dsc+=p.geturl();
	}
	char buf[64];
	snprintf(buf, 64, "%i", p.getpkgsize());
	dsc+="</td></tr><tr><td><b>RPM size: </b></td><td>";
	dsc+=buf;
	snprintf(buf, 64, "%i", p.getsize());
	dsc+="<b> and unpacked: </b>";
	dsc+=buf;
	dsc+="</td></tr></table><br><b>Description:</b>";
	
	list<string> description=p.getdescription();
	list<string>::iterator it;
	for(it=description.begin(); it!=description.end(); it++)
		dsc+="<br>"+*it;
	
	pkgdesc->setText(dsc);
	
	if(p.isinstalled())
	{
		installbutton->setEnabled(false);
		uninstallbutton->setEnabled(true);
	}else{
		installbutton->setEnabled(true);
		uninstallbutton->setEnabled(false);
	}
}

void MainDialog::install()
{
	assert(selpkg);
	if(t)
		return;
	
	wait->text->setText(I18N_NOOP("Checking dependencies..."));
	wait->show();
	QTimer::singleShot(20, mainwin, SLOT(install2()));
}

void MainDialog::install2()
{
	t=new Transaction;
	t->try_command(selpkg->getname(), OP_INSTALL);
	wait->hide();
	
	if(t->get_errors().empty())
	{
		string s;
		list<string> ls;
		
		s+=I18N_NOOP("<b>Need to install following packages:</b><br>");
		ls=t->get_installs();
		for(list<string>::const_iterator i=ls.begin(); i!=ls.end(); i++)
			if(i==ls.begin())
				s+=*i;
			else
				s+=", "+*i;
		ls=t->get_deps();
		for(list<string>::const_iterator i=ls.begin(); i!=ls.end(); i++)
			s+=", "+*i;
		ls=t->get_removes();
		if(!ls.empty())
		{
			s+=I18N_NOOP("<br><b>Need to uninstall following packages:</b><br>");
			for(list<string>::const_iterator i=ls.begin(); i!=ls.end(); i++)
				if(i==ls.begin())
					s+=*i;
				else
					s+=", "+*i;
		}
		
		yesno->text->setText(s);
		yesno->nobutton->disconnect(SIGNAL(clicked()));
		yesno->yesbutton->disconnect(SIGNAL(clicked()));
		connect((QObject*)yesno->nobutton, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
		connect((QObject*)yesno->yesbutton, SIGNAL(clicked()), mainwin, SLOT(install3()));
		yesno->show();
	}else{
		string s;
		list<string> errors=t->get_errors();

		s+=I18N_NOOP("<b><font color=\"red\">Installation cannot proceed because of following errors</font></b>:<br>");
		for(list<string>::const_iterator i=errors.begin(); i!=errors.end(); i++)
		{
			s+=*i;
			s+="<br>";
		}
		
		info->text->setText(s);
		info->button->disconnect(SIGNAL(clicked()));
		connect((QObject*)info->button, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
		info->show();
	}
}

void MainDialog::install3()
{
	yesno->hide();
	wait->text->setText(I18N_NOOP("Installing..."));
	wait->show();
	QTimer::singleShot(20, mainwin, SLOT(install4()));
}

void MainDialog::install4()
{
	list<string> buf;
	string s(I18N_NOOP("<b>Please review poldek output to see if everything went ok:</b><br>"));

	t->start();
	while(!t->eof())
		buf.push_back(t->gets());
	ptree->recheck_installed();
	fill_pkglist(selgrp);
	
	wait->hide();
	
	for(list<string>::const_iterator i=buf.begin(); i!=buf.end(); i++)
		s+=*i+"<br>";
	
	info->text->setText(s);
	info->button->disconnect(SIGNAL(clicked()));
	connect((QObject*)info->button, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
	info->show();
}

void MainDialog::install_cleanup()
{
	yesno->hide();
	info->hide();
	delete t;
	t=NULL;
}

void MainDialog::uninstall()
{
	assert(selpkg);
	if(t)
		return;
	
	wait->text->setText(I18N_NOOP("Checking dependencies..."));
	wait->show();
	QTimer::singleShot(20, mainwin, SLOT(uninstall2()));
}

void MainDialog::uninstall2()
{
	t=new Transaction;
	t->try_command(selpkg->getname(), OP_UNINSTALL);
	wait->hide();
	
	if(t->get_errors().empty())
	{
		string s;
		list<string> ls;
		
		s+=I18N_NOOP("<b>Need to uninstall following packages:</b><br>");
		ls=t->get_removes();
		for(list<string>::const_iterator i=ls.begin(); i!=ls.end(); i++)
			if(i==ls.begin())
				s+=*i;
			else
				s+=", "+*i;
		ls=t->get_deps();
		for(list<string>::const_iterator i=ls.begin(); i!=ls.end(); i++)
			s+=", "+*i;
		
		yesno->text->setText(s);
		yesno->nobutton->disconnect(SIGNAL(clicked()));
		yesno->yesbutton->disconnect(SIGNAL(clicked()));
		connect((QObject*)yesno->nobutton, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
		connect((QObject*)yesno->yesbutton, SIGNAL(clicked()), mainwin, SLOT(uninstall3()));
		yesno->show();
	}else{
		string s;
		list<string> errors=t->get_errors();

		s+=I18N_NOOP("<b><font color=\"red\">Uninstallation cannot proceed because of following errors</font></b>:<br>");
		for(list<string>::const_iterator i=errors.begin(); i!=errors.end(); i++)
		{
			s+=*i;
			s+="<br>";
		}
		
		info->text->setText(s);
		info->button->disconnect(SIGNAL(clicked()));
		connect((QObject*)info->button, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
		info->show();
	}
}

void MainDialog::uninstall3()
{
	yesno->hide();
	wait->text->setText(I18N_NOOP("Uninstalling..."));
	wait->show();
	QTimer::singleShot(20, mainwin, SLOT(uninstall4()));
}

void MainDialog::uninstall4()
{
	list<string> buf;
	string s(I18N_NOOP("<b>Please review poldek output to see if everything went ok:</b><br>"));

	t->start();
	while(!t->eof())
		buf.push_back(t->gets());
	ptree->recheck_installed();
	fill_pkglist(selgrp);
	
	wait->hide();
	
	for(list<string>::const_iterator i=buf.begin(); i!=buf.end(); i++)
		s+=*i+"<br>";
	
	info->text->setText(s);
	info->button->disconnect(SIGNAL(clicked()));
	connect((QObject*)info->button, SIGNAL(clicked()), mainwin, SLOT(install_cleanup()));
	info->show();
}

string MainDialog::getgroupname(QListViewItem *item)
{
	if(!item->parent())
		return item->text(0);
	
	return getgroupname(item->parent())+"/"+item->text(0);
}

#include "maindialog.moc"
